
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:fluttertoast/fluttertoast.dart';
import 'package:linkv_rtc/linkv_rtc_engine.dart';
import 'package:linkv_rtc/linkv_error_code.dart';
import 'package:linkv_rtc/linkv_video_widget.dart';
import 'dart:math';
import 'package:linkv_rtc/linkv_api_defines.dart';
import 'package:linkv_example/linkv_ui.dart';
import 'package:linkv_rtc/linkv_video_callback.dart';

class LivePage extends StatefulWidget {

  final bool isMaster;
  final String roomId;

  LivePage(this.isMaster, this.roomId);

  @override
  State<StatefulWidget> createState() {
    return new _LivePageState();
  }
}

class _LivePageState extends State<LivePage> implements LinkVVideoCallback {

  String _userId;
  String _remoteUid = "";
  LVCameraPosition _position = LVCameraPosition.front;
  
  int _local_width = 0;
  int _local_height = 0;
  int _local_fps = 0;
  
  int _remote_width = 0;
  int _remote_height = 0;
  int _remote_fps = 0;
  
  @override
  void initState() {
    super.initState();
    
    loginRoom();
    LinkVRTCEngine.setVideoCallback(this);
  }
  
  void loginRoom() async {
    _userId = Random().nextInt(1000000).toString();
    print("D/Wing own userId = " + _userId);
    Fluttertoast.showToast(msg: "开始登陆房间", toastLength: Toast.LENGTH_SHORT, gravity: ToastGravity.CENTER);
    var result = await LinkVRTCEngine.loginRoom(widget.roomId, _userId, widget.isMaster, false);
    print("Wing login room result = ${result.code}");
    if (result.code != LVErrorCode.kOK) {
      LinkvUi.showAlert(context, "login room error: ${result.code}");
      return;
    }

    Fluttertoast.showToast(msg: "登陆成功", toastLength: Toast.LENGTH_SHORT, gravity: ToastGravity.CENTER);
    String streamId = "";
    for (LVUser user in result.users) {
      print("D/Wing for user_id = " + user.userId);
      if (user.userId != _userId) {
        streamId = user.userId;
        break;
      }
    }

    if (widget.isMaster) {
      // 是主播，开始推流
      LinkVRTCEngine.startPublishing();
      LinkVRTCEngine.setPublishQualityMonitorCycle(2);
    }

    addRemoteView(streamId);
  }

  void addRemoteView(String streamId) {
    if (streamId == null || streamId.length == 0 || streamId == _remoteUid) return;

    print("addRemoteView streamId = " + streamId);
    setState(() {
      _remoteUid = streamId;
    });
    LinkVRTCEngine.startPlayingStream(streamId);
    LinkVRTCEngine.setPlayQualityMonitor(2);
  }

  @override
  void dispose() {
    super.dispose();

    if (widget.isMaster) {
      LinkVRTCEngine.stopCapture();
      LinkVRTCEngine.stopPublishStream();
    }
    if (_remoteUid.length != 0) {
      LinkVRTCEngine.stopPlayingStream(_remoteUid);
    }

    LinkVRTCEngine.logoutRoom();
  }

  void onClickSwitchCamera() {
    _position = _position == LVCameraPosition.front ? LVCameraPosition.back : LVCameraPosition.front;
    LinkVRTCEngine.switchCamera(_position);
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        appBar: AppBar(title: Text("room_id: ${widget.roomId}")),
        body: Container(
          color: Colors.black26,
          width: MediaQuery.of(context).size.width,
          child: Column(
            children: <Widget>[
              buildLocalView(),
              buildRemoteView(),
              switchCamera(context),
            ],
          ),
        )
    );
  }

  Widget switchCamera(BuildContext context) {
    return Container(
      margin: EdgeInsets.only(top: 20, left: 20, right: 20),
      width: MediaQuery.of(context).size.width - 40,
      height: 50,
      color: Colors.cyan,
      child: CupertinoButton(child: Text("Switch Camera", style: TextStyle(color: Colors.white),), onPressed: onClickSwitchCamera,),
    );
  }

  Widget buildRemoteView() {
    if (_remoteUid.length == 0) return Container();

    return Column(
      children: <Widget>[
        Padding(padding: EdgeInsets.only(top: 10),),
        Stack(
          children: <Widget>[
            Container(
              width: MediaQuery.of(context).size.width,
              height: MediaQuery.of(context).size.height * 0.5 - 120,
              child: LinkVVideoWidget(uid: _remoteUid, isMaster: false),
            ),
            remoteFpsView(),
          ],
        )
      ],
    );
  }

  Widget buildLocalView() {
    if (!widget.isMaster) return Container();
    return Column(
      children: <Widget>[
        Container(
          padding: const EdgeInsets.symmetric(vertical: 10),
          child: Text("streamId: $_userId"),
        ),
        Stack(
          children: <Widget>[
            Container(
              width: MediaQuery.of(context).size.width,
              height: MediaQuery.of(context).size.height * 0.5 - 120,
              child: LinkVVideoWidget(uid: _userId, isMaster: widget.isMaster),
            ),
            localFpsView(),
          ],
        )
      ],
    );
  }
  
  Widget localFpsView() {
    if (_local_width == 0) return Container();
    return Positioned(
      left: 20, top: 10,
      child: Text("${_local_width}x${_local_height} / ${_local_fps}fps \n${_userId}"),
    );
  }

  Widget remoteFpsView() {
    if (_remote_width == 0) return Container();
    return Positioned(
      left: 20, top: 10,
      child: Text("${_remote_width}x${_remote_height} / ${_remote_fps}fps \n${_remoteUid}"),
    );
  }

  // MARK: - Videocallback
  @override
  void onAddRemoter(LVUser lvUser) {
    if (_remoteUid.length != 0) return;

    addRemoteView(lvUser.userId);
  }

  @override
  void onDeleteRemoter(String streamId) {
    if (streamId != null && streamId == _remoteUid) {
      setState(() {
        _remoteUid = "";
      });
      LinkVRTCEngine.stopPlayingStream(streamId);
    }
  }

  @override
  void onDrawFrame(int i, int i1, int i2, String s, String s1) {
    // TODO: implement onDrawFrame
  }

  @override
  void onExitRoomComplete() {
    // TODO: implement onExitRoomComplete
  }

  @override
  void onKickOff(int reason, String roomId) {
    // TODO: implement onKickOff
  }

  @override
  void onMixComplete(bool b) {
    // TODO: implement onMixComplete
  }

  @override
  void onPlayQualityUpdate(LVVideoStatistic lvVideoStatistic, String s) {
    print("Wing onPlayStateUpdate streamId = ${s} fps = ${lvVideoStatistic.videoFps} width = ${lvVideoStatistic.frameWidth}");

    setState(() {
      _remote_fps = lvVideoStatistic.videoFps;
      _remote_width = lvVideoStatistic.frameWidth;
      _remote_height = lvVideoStatistic.frameHeight;
    });
  }

  @override
  void onPlayStateUpdate(int state, String streamId) {
    print("Wing onPlayStateUpdate streamId = ${streamId} state = ${state}");
  }

  @override
  void onPublishQualityUpdate(LVVideoStatistic lvVideoStatistic) {
    print("Wing onPublishQualityUpdate fps = ${lvVideoStatistic.videoFps} width = ${lvVideoStatistic.frameWidth}");

    setState(() {
      _local_fps = lvVideoStatistic.videoFps;
      _local_width = lvVideoStatistic.frameWidth;
      _local_height = lvVideoStatistic.frameHeight;
    });
  }

  @override
  void onPublishStateUpdate(int state) {
    print("Wing onPlayStateUpdate state = ${state}");
  }

  @override
  void onRoomDisconnect(int code) {
    // TODO: implement onRoomDisconnect
  }

  @override
  void onRoomReconnect() {
    // TODO: implement onRoomReconnect
  }

}
